@@ -1,16 +1,16 @@
 #ifndef __GHCPLATFORM_H__
 #define __GHCPLATFORM_H__
 
-#define BuildPlatform_TYPE  i386_unknown_openbsd
-#define HostPlatform_TYPE   i386_unknown_openbsd
+#define BuildPlatform_TYPE  x86_64_unknown_openbsd
+#define HostPlatform_TYPE   x86_64_unknown_openbsd
 
-#define i386_unknown_openbsd_BUILD  1
-#define i386_unknown_openbsd_HOST  1
+#define x86_64_unknown_openbsd_BUILD  1
+#define x86_64_unknown_openbsd_HOST  1
 
-#define i386_BUILD_ARCH  1
-#define i386_HOST_ARCH  1
-#define BUILD_ARCH  "i386"
-#define HOST_ARCH  "i386"
+#define x86_64_BUILD_ARCH  1
+#define x86_64_HOST_ARCH  1
+#define BUILD_ARCH  "x86_64"
+#define HOST_ARCH  "x86_64"
 
 #define openbsd_BUILD_OS  1
 #define openbsd_HOST_OS  1
@@ -23,10 +23,10 @@
 #define HOST_VENDOR  "unknown"
 
 /* These TARGET macros are for backwards compatibility... DO NOT USE! */
-#define TargetPlatform_TYPE i386_unknown_openbsd
-#define i386_unknown_openbsd_TARGET  1
-#define i386_TARGET_ARCH  1
-#define TARGET_ARCH  "i386"
+#define TargetPlatform_TYPE x86_64_unknown_openbsd
+#define x86_64_unknown_openbsd_TARGET  1
+#define x86_64_TARGET_ARCH  1
+#define TARGET_ARCH  "x86_64"
 #define openbsd_TARGET_OS  1
 #define TARGET_OS  "openbsd"
 #define unknown_TARGET_VENDOR  1
